#!/bin/bash

[[ $1 = -v ]] && VERBOSE=1 || unset VERBOSE
[[ -z $VERBOSE ]] && exec 2>/dev/null
export VERBOSE

if [[ ! -w /etc/passwd ]] ; then
  echo "Only 'root' can execute this script."
  exit 1
fi

#
# Set epages environmental variables
#
SYS_SCRIPT=/etc/sysconfig/epages6
[[ -f $SYS_SCRIPT ]] && . $SYS_SCRIPT
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

die() {
  touch $EPAGES/Setup/mysql.ini
  exit 0
}

unset VOID_VAR
void_var() {
  echo "Following variable value(s) may be incorrect:"
  for i in $@ ; do echo "  $i=\"$(eval echo '$'$i)\"" ; done
  echo "Check these variable(s) in $SYS_SCRIPT and rerun:"
  echo "  $0"
  VOID_VAR=1
}

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

RUN_SCRIPT=/etc/default/$EPAGES_PROJECT
ls $RUN_SCRIPT >/dev/null || die
. $RUN_SCRIPT

export get_mysql_option get_mysql_connection set_mysql_connection my_cnf
export EPAGES_INSTALL_MY_CNF

#------------------------------------------------------------------------------
# run
#------------------------------------------------------------------------------

#
# Read the connection settings from sysconfig file
#
CONN=($(get_mysql_connection))
MYSQL_HOST=${CONN[0]} ; MYSQL_PORT=${CONN[1]}

#
# Set the mysql connection string
#
if [[ $MYSQL_PORT = 3306 ]] ; then
  EPAGES_INSTALL_MYSQL_CONNECTION=$MYSQL_HOST
  MYSQL_CONN="host=$MYSQL_HOST"
  CONNECTION_ARGS="-h $MYSQL_HOST"
else
  EPAGES_INSTALL_MYSQL_CONNECTION="$MYSQL_HOST;$MYSQL_PORT"
  MYSQL_CONN="host=$MYSQL_HOST;port=$MYSQL_PORT"
  CONNECTION_ARGS="-h $MYSQL_HOST -P $MYSQL_PORT"
fi

#
# Read the backup user credentials
#
unset MYSQL_PASS_ARG
export MYSQL_USER=${EPAGES_INSTALL_MYSQL_USER:-root}
export MYSQL_PASS=$EPAGES_INSTALL_MYSQL_PASS
MYSQL_PASS_ARG=${MYSQL_PASS:+"-p'$MYSQL_PASS'"}

#
# Test the socket connection
#
check_mysql_connection(){
  local RET
  if [[ ${@%-pepages} = $@ ]] ; then
    [[ -n $VERBOSE ]] && echo "echo exit | eval mysql $@"
    echo "mysql $@" >/tmp/proc.~$$~
    echo exit | eval mysql $@ 2>>/tmp/proc.~$$~
    RET=$? 
    [[ -n $VERBOSE ]] && echo "-> $(cat /tmp/proc.~$$~)"
  else
    [[ -n $VERBOSE ]] && echo "echo exit | eval mysql $@"
    echo exit | eval mysql $@
    RET=$?
    [[ $RET = 0 ]] && { MYSQL_PASS=epages; MYSQL_PASS_ARG=-pepages; }
  fi
  return $RET
}

MYSQL_SECTIONS='mysqld server mysql_server mysql.server mysqld_safe client mysql'
MYSQL_DATADIR=$(get_mysql_option "$MYSQL_SECTIONS" datadir /var/lib/mysql)
MYSQL_SOCKET=$(get_mysql_option "$MYSQL_SECTIONS" socket $MYSQL_DATADIR/mysql.sock)
MYSQL_SOCKET_ARG=-S$MYSQL_SOCKET

#
# Reset password if root has random password (MySQL 5.6)
#
if [[ $MYSQL_USER = root && -f /root/.mysql_secret ]] ; then
  pass=$(sed 's,.*:,,' /root/.mysql_secret) ; pass=${pass// /}
  if [[ -n $pass ]] ; then

    # set password for root
    [[ -n $VERBOSE ]] && echo "mysqladmin $MYSQL_SOCKET_ARG -u root -p'$pass' password '$MYSQL_PASS'"
    mysqladmin $MYSQL_SOCKET_ARG -uroot -p"$pass" password "$MYSQL_PASS"
    sleep 1

    if check_mysql_connection $MYSQL_SOCKET_ARG -uroot $MYSQL_PASS_ARG ; then
      SELECT="SELECT host FROM mysql.user WHERE user = 'root';"
      echo "$SELECT" | eval mysql $MYSQL_SOCKET_ARG -uroot -rs $MYSQL_PASS_ARG 2>/dev/null \
      | while read host ; do
        # set password for root @ $host
        STATEMENT="SET PASSWORD FOR 'root'@'$host' = PASSWORD('$MYSQL_PASS');"
        [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u root $MYSQL_PASS_ARG"
        echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u root $MYSQL_PASS_ARG || {
          [[ -z $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u root $MYSQL_PASS_ARG"
          die
        }
      done
    fi

  fi
fi

if check_mysql_connection $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG \
   || check_mysql_connection $MYSQL_SOCKET_ARG -u$MYSQL_USER -pepages ; then
  rm -f /tmp/proc.~$$~
  mv /root/.mysql_secret /root/.mysql_secret.bak
else
  [[ -z $VERBOSE ]] && cat /tmp/proc.~$$~
  void_var EPAGES_INSTALL_MYSQL_CONNECTION EPAGES_INSTALL_MYSQL_USER EPAGES_INSTALL_MYSQL_PASS
fi

#
# Database engine must be InnoDB
#
if [[ -z $VOID_VAR ]] ; then
  STATEMENT="SHOW ENGINES;"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG | grep -Pq '^InnoDB\\s+(YES|DEFAULT)\\s+'"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG | grep -Pq '^InnoDB\s+(YES|DEFAULT)\s+' || {
    [[ -z $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG | grep -Pq '^InnoDB\\s+(YES|DEFAULT)\\s+'"
    echo "ERROR: Statement failed, database engine InnoDB not enabled."
    die
  }
fi

#
# Grant all privileges on *.* to $MYSQL_USER
#
if [[ -z $VOID_VAR ]] ; then
  STATEMENT="DELETE FROM mysql.user WHERE user = ''; FLUSH PRIVILEGES;"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG
  STATEMENT="SELECT user,host FROM mysql.user WHERE user = '';"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG > /tmp/pid.$$
  if [[ -s /tmp/pid.$$ ]] ; then
    [[ -z $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
    echo "ERROR: Statement failed, empty users still exist:"
    cat /tmp/pid.$$
    rm -f /tmp/pid.$$
    die
  fi
  rm -f /tmp/pid.$$
  [[ -z $MYSQL_PASS ]] && IDENTIFIED_BY= || IDENTIFIED_BY="IDENTIFIED BY '$MYSQL_PASS'"
  STATEMENT="GRANT ALL ON *.* TO '$MYSQL_USER' $IDENTIFIED_BY WITH GRANT OPTION;"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG
fi

#
# Unbind address
#
MY_CNF=${EPAGES_INSTALL_MY_CNF:-$(my_cnf)}
if [[ -f $MY_CNF ]] ; then
  sed -r 's,^\s*(bind-address),#\1,' $MY_CNF > $MY_CNF.tmp
  if ! cmp -s $MY_CNF $MY_CNF.tmp ; then
    cat $MY_CNF.tmp > $MY_CNF
    service $(mysqld_name) restart
    sleep 2
  fi
  rm -f $MY_CNF.tmp
fi

#
# Test the connection based on the connection string
#
if check_mysql_connection $CONNECTION_ARGS -u$MYSQL_USER $MYSQL_PASS_ARG ; then
  rm -f /tmp/proc.~$$~
else
  [[ -z $VERBOSE ]] && cat /tmp/proc.~$$~
  void_var EPAGES_INSTALL_MYSQL_CONNECTION EPAGES_INSTALL_MYSQL_USER EPAGES_INSTALL_MYSQL_PASS
fi

#
# Set EPAGES_MYSQL_CONNECTION in /etc/default/epages6
#
INI_CONFIG=/var/epages/ini-config.sh
if [[ $EPAGES_MYSQL_CONNECTION != $EPAGES_INSTALL_MYSQL_CONNECTION ]] ; then
  [[ -n $VERBOSE ]] && echo "$INI_CONFIG -f $RUN_SCRIPT -k EPAGES_MYSQL_CONNECTION -v '\"$EPAGES_INSTALL_MYSQL_CONNECTION\"' -i"
  $INI_CONFIG -f $RUN_SCRIPT -k EPAGES_MYSQL_CONNECTION -v "\"$EPAGES_INSTALL_MYSQL_CONNECTION\"" -i
fi

if [[ $EPAGES_INSTALL_MYSQL_HOST != $MYSQL_HOST ]] ; then
  [[ -n $VERBOSE ]] && echo "$INI_CONFIG -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_HOST -v '\"$MYSQL_HOST\"' -i"
  $INI_CONFIG -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_HOST -v "\"$MYSQL_HOST\"" -i
fi
[[ -n $VERBOSE ]] && echo "$INI_CONFIG -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_CONNECTION -v '\"$EPAGES_INSTALL_MYSQL_CONNECTION\"' -i"
$INI_CONFIG -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_CONNECTION -v "\"$EPAGES_INSTALL_MYSQL_CONNECTION\"" -i

[[ -z $VOID_VAR ]] || rm -f $EPAGES/Setup/mysql.ini

#
# Rewrite Backup.conf to update the datasource string and the backup user credentials
#
DB_CONF="$EPAGES_CONFIG/Database.d/Backup.conf"
if [[ -f $DB_CONF ]] ; then

  $INI_CONFIG -f $DB_CONF -s Connection -k login -v "$MYSQL_USER" -i
  $INI_CONFIG -f $DB_CONF -s Connection -k datasource -v "dbi:mysql:$MYSQL_CONN" -i
  
  ENC=$($PERL -e "use DE_EPAGES::Database::API::Connection qw(EncryptDBPassword); print EncryptDBPassword('$MYSQL_PASS')")
  [[ -z $ENC ]] && { USE=passwd; DROP=encpasswd; } || { USE=encpasswd; DROP=passwd; MYSQL_PASS=$ENC; }

  grep -v $DROP $DB_CONF > $DB_CONF.$$ && cat $DB_CONF.$$ > $DB_CONF
  rm -f $DB_CONF.$$
  $INI_CONFIG -f $DB_CONF -s Connection -k $USE -v "$MYSQL_PASS" -i

  if [[ -n $EPAGES_INSTALL_MY_CNF ]] ; then
    $INI_CONFIG -f $DB_CONF -s Connection -k MySQLConfigFile -v "$EPAGES_INSTALL_MY_CNF" -i
  fi

  chown ep_appl:epages $DB_CONF
fi

#
# Write ASPoolDB.conf unless it contains datasource=dbi:mysql:host=
#
DB_CONF="$EPAGES_CONFIG/Database.d/ASPoolDB.conf"
if [[ -f  $DB_CONF.sample.initial ]] ; then
  cp -p $DB_CONF.sample.initial $DB_CONF.initial
fi
[[ ! -f $DB_CONF ]] && cp -p $DB_CONF.initial $DB_CONF

if ! grep -q 'datasource=dbi:mysql:host=' $DB_CONF ; then
  set_mysql_connection ASPoolDB $(get_mysql_connection)
fi

exit 0
